// Using an IIFE to create an object with some functionality

var adder = (function() {
	return {
		add: function(x, y) { return x + y; }
	};
}());

console.log("Adding 5 and 3: " + adder.add(5, 3));

















// -----------------------------------------------------
// Re-using the global adder to construct a calculator

var firstCalculator = (function() {
	return {
		add: function(x, y) { return adder.add(x, y); }
	};
}());

console.log("Adding with the firstCalculator: " + 
	firstCalculator.add(5, 3));















// -----------------------------------------------------
// Better to pass in the adder - cleaner and faster

var improvedCalculator = (function(adder) {

	return {
		add: function(x, y) { return adder.add(x, y); }
	};
}(adder));

console.log("Adding with the improvedCalculator: " +
	improvedCalculator.add(5, 3));











// Btw - this is the kind of thing you often see with 
// "standard" global
// objects like jQuery...

// var someModule = (function($) {
// 	// Can now use jQuery in here with standard $ syntax

// 	return {
// 		//...
// 	};
// }(jQuery));
















// -----------------------------------------------------
// Construct the object in a way that makes it possible to 
// augment later - or to execute as an augmentation in the first place.

var calculator = (function(me) {
	me.mult = function(x, y) { return x * y; };
	return me;
}(calculator || {}));

console.log("Multiplying with the augmented calculator: " + calculator.mult(5, 3));

var calculator = (function(me, adder) {
	me.add = adder.add; 
	return me;
}(calculator || {}, adder));

console.log("Adding with the even better calculator: " + calculator.add(5, 3));


console.log("Adding with the complete calculator: " + calculator.add(5,3));






// The order of initialization above is now irrelevant - try it!













// -----------------------------------------------------
// We can also change an existing piece of functionality by augmenting,
// but that means the initialization order is now relevant.
// See the last line - the {} empty instance is not an option now.
// Sometimes this is called "tight augmentation".

var calculator = (function(me) {
	var standardMult = me.mult;
	me.mult = function(x, y) { return x * 3; }; // for some reason...
	return me;
}(calculator));

console.log("Multiplying with the weird * 3 augmented calculator: " + 
	calculator.mult(7, 2));














// -----------------------------------------------------
// Let's augment something with state this time

var calculator = (function(me) {
	var innerstore = 0;
	me.store = function(x) { 
		innerstore = x; 
	};
	me.retrieve = function() { 
		return innerstore; 
	};
	me.storeAdd = function(x) { 
		innerstore += x; 
	};
	me.storeSubtract = function(x) { 
		innerstore -= x; 
	};
	me.storeReset = function(x) { 
		me.store(0); 
	};

	return me;
}(calculator || {}));

console.log("Storing value 42...");
calculator.store(42);
console.log("Stored value is: " + calculator.retrieve());

